@@ -7,8 +7,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef _MSC_VER
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -42,7 +44,6 @@ extern char _STR[150];
 extern Globaldata xobjs;
 extern XCWindowData *areawin;
 extern Display *dpy;
-extern int *appcolors;
 
 /*----------------------------------------------*/
 #ifdef LGF
@@ -87,7 +88,7 @@ void loadlgf(int mode)
 
    /* Make sure that LGF object library has been loaded by loading it now. */
 
-   if (check_library(LGF_LIB) < 0) {
+   if (NameToLibrary(LGF_LIB) < 0) {
       int ilib;
       strcpy(_STR, LGF_LIB);
       ilib = createlibrary(FALSE);
@@ -232,7 +233,6 @@ void loadlgf(int mode)
 		  stringpart *strptr;
 
 		  NEW_LABEL(newlabel, topobject);
-	          topobject->parts++;
 
 		  labeldefaults(*newlabel, False, xmat(pvalx), ymat(pvaly));
 	          (*newlabel)->justify = TOP | NOTBOTTOM;
@@ -264,7 +264,6 @@ void loadlgf(int mode)
 	       /* Allocate wire */
 
 	       NEW_POLY(newwire, topobject);
-	       topobject->parts++;
 
 	       sscanf(lineptr, "%hd %hd %hd %hd", &pvalx, &pvaly, &pvalx2, &pvaly2);
 	       (*newwire)->number = 2;
@@ -315,7 +314,6 @@ void loadlgf(int mode)
 	       } while (*lineptr == '\n');
 
 	       NEW_POLY(newpoly, topobject);
-	       topobject->parts++;
 
 	       (*newpoly)->style = DASHED;
 	       (*newpoly)->color = DEFAULTCOLOR;
@@ -377,7 +375,6 @@ void loadlgf(int mode)
 	       if (j < xobjs.numlibs || k < xobjs.userlibs[xobjs.numlibs - 1].number) {
 
 		  NEW_OBJINST(newinst, topobject);
-      	          topobject->parts++;
 
 	          sscanf(lineptr, "%hd %hd %hd %*d %*d %*d %d", &pvalx, &pvaly,
 			&pvalx2, &hval); 
@@ -415,7 +412,6 @@ void loadlgf(int mode)
 			   stringpart *strptr;
 
 			   NEW_LABEL(newlabel, topobject);
-			   topobject->parts++;
 			   /* reconnect newinst if displaced by realloc() */
 			   newinst = (objinstptr *)(topobject->plist
 				+ topobject->parts - 2);
@@ -423,10 +419,8 @@ void loadlgf(int mode)
 			   labeldefaults(*newlabel, False, (*newinst)->position.x,
 				(*newinst)->position.y);
 			   (*newlabel)->color = DEFAULTCOLOR;
-		  	   if (areawin->schemon){
-			      (*newlabel)->pin = LOCAL;
-			      (*newlabel)->color = LOCALPINCOLOR;
-		  	   }
+			   (*newlabel)->pin = LOCAL;
+			   (*newlabel)->color = LOCALPINCOLOR;
 			   if (!strcmp(tmpstring, "TO"))
 			      (*newlabel)->position.x += ((flip) ? 48 : -48);
 			   else
@@ -635,7 +629,6 @@ void loadmat4(caddr_t nullval)
 	       sscanf(bb + 1, "%f %f %f", &red, &green, &blue);
 	       curcolor = rgb_alloccolor((int)(red * 65535), (int)(green * 65535),
 			(int)(blue * 65535));
-	       addnewcolorentry(curcolor);	
 	    }
 	    if ((bb = strchr(buffer, '/')) != NULL) {
 	       sscanf(bb, "%4s", &colorstr[matcolors]);
