@@ -42,7 +42,8 @@ extern Display *dpy;
 extern Pixmap STIPPLE[8];
 extern XCWindowData *areawin;
 extern Globaldata xobjs;
-extern int *appcolors;
+extern int number_colors;
+extern colorindex *colorlist;
 
 /*------------------------------------------------------------------------*/
 /* find the squared length of a wire (or distance between two points in   */
@@ -893,12 +894,13 @@ void UMakeWCTM(Matrix *ctm)
 {
    ctm->a *= areawin->vscale;
    ctm->b *= areawin->vscale;
-   ctm->c = (ctm->c - (float)areawin->pcorner.x) * areawin->vscale;
+   ctm->c = (ctm->c - (float)areawin->pcorner.x) * areawin->vscale
+	 + areawin->panx;
    
    ctm->d *= -areawin->vscale;
    ctm->e *= -areawin->vscale;
    ctm->f = (float)areawin->height + ((float)areawin->pcorner.y - ctm->f) *
-	    areawin->vscale;
+	 areawin->vscale + areawin->pany;
 
 #ifdef HAVE_CAIRO
    if (ctm == DCTM && areawin->redraw_ongoing)
@@ -1826,6 +1828,7 @@ void centerview(objinstptr tinst)
 
 void refresh(xcWidget bw, caddr_t clientdata, caddr_t calldata)
 {
+   areawin->redraw_needed = True;
    drawarea(NULL, NULL, NULL);
    if (areawin->scrollbarh)
       drawhbar(areawin->scrollbarh, NULL, NULL);
@@ -1861,6 +1864,11 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
 {
    XPoint newpt1, newpt2;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    UTransformbyCTM(DCTM, pt1, &newpt1, 1);
    UTransformbyCTM(DCTM, pt2, &newpt2, 1);
 
@@ -1876,6 +1884,11 @@ void UDrawLine(XPoint *pt1, XPoint *pt2)
 {
    float tmpwidth = UTopTransScale(xobjs.pagelist[areawin->page]->wirewidth);
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid, CapRound, JoinBevel);
    UDrawSimpleLine(pt1, pt2); 
 } 
@@ -1900,6 +1913,11 @@ void UDrawCircle(XPoint *upt, u_char which)
 {
    XPoint wpt;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    user_to_window(*upt, &wpt);
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapButt, JoinMiter);
 
@@ -1931,6 +1949,11 @@ void UDrawCircle(XPoint *upt, u_char which)
 #ifndef HAVE_CAIRO
 void UDrawXAt(XPoint *wpt)
 {
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapButt, JoinMiter);
    DrawLine(dpy, areawin->window, areawin->gc, wpt->x - 3,
 		wpt->y - 3, wpt->x + 3, wpt->y + 3);
@@ -2065,6 +2088,11 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    XPoint  points[2]; /* top and bottom of text cursor line */
    short   xdist, xbase, tmpjust;
    TextExtents tmpext;
+   
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
 
    /* correct for position, rotation, scale, and flip invariance of text */
 
@@ -2072,12 +2100,7 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    UPreMultCTM(DCTM, curlabel->position, curlabel->scale, curlabel->rotation);
    tmpjust = flipadjust(curlabel->justify);
 
-#ifdef HAVE_CAIRO
    SetForeground(dpy, areawin->gc, AUXCOLOR);
-#else /* HAVE_CAIRO */
-   SetFunction(dpy, areawin->gc, GXxor);
-   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
-#endif /* HAVE_CAIRO */
 
    tmpext = ULength(curlabel, areawin->topinstance, tpos, NULL);
    xdist = tmpext.width;
@@ -2127,8 +2150,12 @@ void UDrawXLine(XPoint opt, XPoint cpt)
 {
    XPoint upt, vpt;
 
-   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
-   SetFunction(dpy, areawin->gc, GXxor);
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
+   SetForeground(dpy, areawin->gc, AUXCOLOR);
 
    user_to_window(cpt, &upt);
    user_to_window(opt, &vpt);
@@ -2142,7 +2169,6 @@ void UDrawXLine(XPoint opt, XPoint cpt)
    DrawLine(dpy, areawin->window, areawin->gc, upt.x + 3, upt.y - 3,
 		upt.x - 3, upt.y + 3);
 
-   SetFunction(dpy, areawin->gc, areawin->gctype);
    SetForeground(dpy, areawin->gc, areawin->gccolor);
 }
 #endif /* HAVE_CAIRO */
@@ -2154,11 +2180,15 @@ void UDrawBox(XPoint origin, XPoint corner)
 {
    XPoint	worig, wcorn;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   SetFunction(dpy, areawin->gc, GXxor);
-   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   SetForeground(dpy, areawin->gc, AUXCOLOR);
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
    DrawLine(dpy, areawin->window, areawin->gc, worig.x, worig.y,
 		worig.x, wcorn.y);
@@ -2186,6 +2216,11 @@ float UGetRescaleBox(XPoint *corner, XPoint *newpoints)
    objinstptr	rinst;
    int		i;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    if (areawin->selects == 0) return 0.0;
 
    /* Use only the 1st selection as a reference to set the scale */
@@ -2299,22 +2334,25 @@ float UGetRescaleBox(XPoint *corner, XPoint *newpoints)
 /*----------------------------------------------------------------------*/
 
 #ifndef HAVE_CAIRO
-float UDrawRescaleBox(XPoint *corner)
+void UDrawRescaleBox(XPoint *corner)
 {
    XPoint origpoints[5], newpoints[5];
-   float newscale;
 
-   if (areawin->selects == 0) return 0.0;
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
 
-   newscale = UGetRescaleBox(corner, newpoints);
+   if (areawin->selects == 0)
+      return;
 
-   SetFunction(dpy, areawin->gc, GXxor);
-   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   UGetRescaleBox(corner, newpoints);
+
+   SetForeground(dpy, areawin->gc, AUXCOLOR);
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
    
    UTransformbyCTM(DCTM, newpoints, origpoints, 4);
    strokepath(origpoints, 4, 0, 1);
-   return newscale;
 }
 #endif /* HAVE_CAIRO */
 
@@ -2327,6 +2365,11 @@ void UDrawBBox()
    XPoint	worig, wcorn, corner;
    objinstptr	bbinst = areawin->topinstance;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    if ((!areawin->bboxon) || (checkforbbox(topobject) != NULL)) return;
 
    origin = bbinst->bbox.lowerleft;
@@ -2471,14 +2514,15 @@ void UDrawSpline(splineptr thespline, float passwidth)
    XPoint       tmppoints[SPLINESEGS];
    float	scaledwidth;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    scaledwidth = thespline->width * passwidth;
 
    makesplinepath(thespline, tmppoints);
    strokepath(tmppoints, SPLINESEGS, thespline->style, scaledwidth);
-   if (thespline->cycle != NULL) {
-      UDrawXLine(thespline->ctrl[0], thespline->ctrl[1]);  
-      UDrawXLine(thespline->ctrl[3], thespline->ctrl[2]);
-   }
 }
 #endif /* HAVE_CAIRO */
 
@@ -2490,6 +2534,11 @@ void UDrawPolygon(polyptr thepoly, float passwidth)
    XPoint *tmppoints = (pointlist) malloc(thepoly->number * sizeof(XPoint));
    float scaledwidth;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    scaledwidth = thepoly->width * passwidth;
 
    UTransformbyCTM(DCTM, thepoly->points, tmppoints, thepoly->number);
@@ -2506,13 +2555,15 @@ void UDrawArc(arcptr thearc, float passwidth)
    XPoint  tmppoints[RSTEPS + 2];
    float scaledwidth;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    scaledwidth = thearc->width * passwidth;
 
    UfTransformbyCTM(DCTM, thearc->points, tmppoints, thearc->number);
    strokepath(tmppoints, thearc->number, thearc->style, scaledwidth);
-   if (thearc->cycle != NULL) {
-      UDrawXLine(thearc->position, areawin->save);
-   }
 }
 #endif /* HAVE_CAIRO */
 
@@ -2527,22 +2578,10 @@ void UDrawPath(pathptr thepath, float passwidth)
    splineptr	thespline;
    int		pathsegs = 0, curseg = 0;
    float	scaledwidth;
-   Boolean	draweditlines = FALSE;
    
-   /* First pass---check for any splines that are being edited.  If	*/
-   /* any one is, then draw all control points for all splines in the	*/
-   /* path.								*/
-
-   for (genpath = thepath->plist; genpath < thepath->plist + thepath->parts;
-	  genpath++) {
-      switch(ELEMENTTYPE(*genpath)) {
-	 case SPLINE:
-	    thespline = TOSPLINE(genpath);
-	    if (thespline->cycle != NULL)
-	       draweditlines = TRUE;
-	    break;
-      }
-      if (draweditlines) break;
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
    }
 
    for (genpath = thepath->plist; genpath < thepath->plist + thepath->parts;
@@ -2561,11 +2600,6 @@ void UDrawPath(pathptr thepath, float passwidth)
 	    tmppoints = (pointlist) realloc(tmppoints, pathsegs * sizeof(XPoint));
    	    makesplinepath(thespline, tmppoints + curseg);
 	    curseg = pathsegs;
-
-	    if (draweditlines) {
-	       UDrawXLine(thespline->ctrl[0], thespline->ctrl[1]);  
-	       UDrawXLine(thespline->ctrl[3], thespline->ctrl[2]);
-	    }
 	    break;
       }
    } 
@@ -2600,6 +2634,11 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
    u_char	xm, ym;
    objectptr	theobject = theinstance->thisobject;
 
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+
    /* Save the number of selections and set it to zero while we do the	*/
    /* object drawing.							*/
 
