@@ -87,7 +87,6 @@ extern short beeper;
 extern Widget menuwidgets[];
 extern Display *dpy;
 extern keybinding *keylist;
-extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
 extern ApplicationData appdata;
@@ -244,7 +243,6 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
 		"newelement() 2nd argument must be a valid element type");
          return NULL;
    } 
-   topobject->parts++;
    incr_changes(topobject);
    invalidate_netlist(topobject);
 
