@@ -4,11 +4,8 @@
 /* These routines work only if ghostscript is on the system.		*/
 /*----------------------------------------------------------------------*/
 
-/* We use OpenGL functions to read and scale images if OpenGL is enabled */
-#ifndef OPENGL
-
-/* #undef GS_DEBUG */
-#define GS_DEBUG
+#undef GS_DEBUG
+/* #define GS_DEBUG */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -59,30 +56,28 @@ extern char _STR2[250], _STR[150];
 extern Globaldata xobjs;
 extern XCWindowData *areawin;
 extern Display *dpy;
-extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
 extern Cursor appcursors[NUM_CURSORS];
 
+#ifdef HAVE_CAIRO
+cairo_surface_t *bbuf = NULL;
+#else /* HAVE_CAIRO */
 Pixmap bbuf = (Pixmap)NULL;	/* background buffer */
-#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
 extern Pixmap dbuf;
-#endif
+#endif /* HAVE_CAIRO */
 
 #ifndef HAVE_VFORK
 #define vfork fork
 #endif
 
-int gs_state;		/* Track state of the renderer */
-
-#define GS_INIT 0	/* Initial state; gs is idle. */
-#define GS_PENDING 1	/* Drawing in progress; gs is busy. */
-#define GS_READY 2	/* Drawing done; gs is waiting for "next". */
+gs_state_t  gs_state;	/* Track state of the renderer */
 
 /*------------------------------------------------------*/
 /* Global variable definitions				*/
 /*------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 Atom  gv, gvc, gvpage, gvnext, gvdone;
 #ifndef _MSC_VER
 pid_t gsproc = -1;	/* ghostscript process 			*/
@@ -93,6 +88,7 @@ int   fgs[2];		/* stdin pipe pair for ghostscript	*/
 Window mwin = 0;	/* "false" window hack to get gs	*/
 			/* process to capture ClientMessage	*/
 			/* events.				*/
+#endif /* HAVE_CAIRO */
 
 /*--------------------------------------------------------------*/
 /* Preliminary in allowing generic PostScript backgrounds 	*/
@@ -103,6 +99,7 @@ Window mwin = 0;	/* "false" window hack to get gs	*/
 
 void ghostinit_local()
 {
+#ifndef HAVE_CAIRO
    sprintf(_STR, "%ld %d %d %d %d %d %g %g %d %d %d %d",
 		0L, 0, 0, 0,
 		areawin->width * 75 / 72,
@@ -114,6 +111,7 @@ void ghostinit_local()
    XChangeProperty(dpy, areawin->window, gvc, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    XSync(dpy, False);
+#endif /* HAVE_CAIRO */
 
    gs_state = GS_INIT;
 }
@@ -129,6 +127,7 @@ void ghostinit_local()
 
 Boolean render_client(XEvent *eventPtr)
 {
+#ifndef HAVE_CAIRO
    if (eventPtr->xclient.message_type == gvpage) {
 #ifdef GS_DEBUG
       fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
@@ -160,6 +159,7 @@ Boolean render_client(XEvent *eventPtr)
       }
       return False;
    }
+#endif /* HAVE_CAIRO */
    return True;
 }
 
@@ -169,11 +169,13 @@ Boolean render_client(XEvent *eventPtr)
 
 #ifdef TCL_WRAPPER
 
+#ifndef HAVE_CAIRO
 void handle_client(ClientData clientData, XEvent *eventPtr)
 {
    if (render_client(eventPtr) == False)
       fprintf(stderr, "Xcircuit: Received unknown client message\n");
 }
+#endif /* HAVE_CAIRO */
 
 #endif
 
@@ -183,6 +185,7 @@ void handle_client(ClientData clientData, XEvent *eventPtr)
 
 void ghostinit()
 {
+#ifndef HAVE_CAIRO
    gv = XInternAtom(dpy, "GHOSTVIEW", False);
    gvc = XInternAtom(dpy, "GHOSTVIEW_COLORS", False);
    gvpage = XInternAtom(dpy, "PAGE", False);
@@ -194,12 +197,14 @@ void ghostinit()
 #ifdef TCL_WRAPPER
    Tk_CreateClientMessageHandler((Tk_ClientMessageProc *)handle_client);
 #endif
+#endif /* HAVE_CAIRO */
 }
 
 /*------------------------------------------------------*/
 /* Send a ClientMessage event				*/
 /*------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void send_client(Atom msg)
 {
    XEvent event;
@@ -217,6 +222,7 @@ void send_client(Atom msg)
    XSendEvent(dpy, mwin, False, 0, &event);
    XFlush(dpy);
 }
+#endif /* HAVE_CAIRO */
 
 /*------------------------------------------------------*/
 /* Ask ghostscript to produce the next page.		*/
@@ -224,6 +230,7 @@ void send_client(Atom msg)
 
 void ask_for_next()
 {
+#ifndef HAVE_CAIRO
    if (gs_state != GS_READY) {
       /* If we're in the middle of rendering something, halt	*/
       /* immediately and start over.				*/
@@ -237,6 +244,7 @@ void ask_for_next()
 #ifdef GS_DEBUG
    fprintf(stdout, "Xcircuit: Sent NEXT message to ghostscript\n");
 #endif
+#endif /* HAVE_CAIRO */
 }
 
 /*--------------------------------------------------------*/
@@ -245,9 +253,9 @@ void ask_for_next()
 /* to xcircuit's stderr)				  */
 /*--------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void start_gs()
 {
-#ifndef HAVE_CAIRO
    int std_out[2], std_err[2], ret;
 #ifdef XC_WIN32
    unsigned int pipe_size = 8196;
@@ -259,25 +267,12 @@ void start_gs()
 
 #ifdef TCL_WRAPPER
    if (bbuf != (Pixmap)NULL) Tk_FreePixmap(dpy, bbuf);
-#ifdef DOUBLEBUFFER
    bbuf = Tk_GetPixmap(dpy, dbuf, areawin->width, areawin->height,
 		Tk_Depth(areawin->area));
-#else
-   bbuf = Tk_GetPixmap(dpy, (Drawable)areawin->window,
-		areawin->width, areawin->height,
-		Tk_Depth(areawin->area));
-#endif
-
 #else	/* !TCL_WRAPPER */
    if (bbuf != (Pixmap)NULL) XFreePixmap(dpy, bbuf);
-#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
    bbuf = XCreatePixmap(dpy, dbuf, areawin->width, areawin->height,
 		DefaultDepthOfScreen(xcScreen(areawin->area))); 
-#else
-   bbuf = XCreatePixmap(dpy, areawin->window,
-		areawin->width, areawin->height,
-		DefaultDepthOfScreen(xcScreen(areawin->area))); 
-#endif	/* DOUBLEBUFFER */
 #endif  /* TCL_WRAPPER */
 
    XSync(dpy, False);
@@ -311,22 +306,12 @@ void start_gs()
 #ifdef HAVE_PUTENV
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
-#ifdef DOUBLEBUFFER
    	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areawin->window, (long)bbuf);
 	 putenv(env_str2);
-#else
-	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areawin->window);
-	 putenv(env_str2);
-#endif
 #else
 	 setenv("DISPLAY", XDisplayString(dpy), True);
-#ifdef DOUBLEBUFFER
 	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
 	 setenv("GHOSTVIEW", _STR, True);
-#else
- 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
-	 setenv("GHOSTVIEW", _STR, True);
-#endif
 #endif
 
 	 SetHandleInformation((HANDLE)_get_osfhandle(fgs[1]), HANDLE_FLAG_INHERIT, 0);
@@ -376,22 +361,12 @@ void start_gs()
 #ifdef HAVE_PUTENV
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
-#ifdef DOUBLEBUFFER
    	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areawin->window, (long)bbuf);
 	 putenv(env_str2);
-#else
-	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areawin->window);
-	 putenv(env_str2);
-#endif
 #else
 	 setenv("DISPLAY", XDisplayString(dpy), True);
-#ifdef DOUBLEBUFFER
 	 sprintf(_STR, "%ld %ld", (long)areawin->window, (long)bbuf);
 	 setenv("GHOSTVIEW", _STR, True);
-#else
- 	 sprintf(_STR, "%ld", (long)areawin->window);
-	 setenv("GHOSTVIEW", _STR, True);
-#endif
 #endif
 	 Flush(stderr);
          execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", (char *)NULL);
@@ -405,8 +380,8 @@ void start_gs()
       }
    }
 #endif
-#endif /* HAVE_CAIRO */
 }
+#endif /* HAVE_CAIRO */
 
 /*--------------------------------------------------------*/
 /* Parse the background file for Bounding Box information */
@@ -580,10 +555,12 @@ void savebackground(FILE *fo, char *psfilename)
 
 void register_bg(char *gsfile)
 {
+#ifndef HAVE_CAIRO
    if (gsproc < 0)
       start_gs();
    else
       reset_gs();
+#endif /* !HAVE_CAIRO */
 
    xobjs.pagelist[areawin->page]->background.name =
 		(char *) malloc(strlen(gsfile) + 1);
@@ -610,6 +587,7 @@ void loadbackground()
 /* Send text to the ghostscript renderer		*/
 /*------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void send_to_gs(char *text)
 {
 #ifndef XC_WIN32
@@ -622,6 +600,33 @@ void send_to_gs(char *text)
    fprintf(stdout, "writing: %s", text);
 #endif
 }
+#endif /* !HAVE_CAIRO */
+
+/*------------------------------------------------------*/
+/* write scale and position to ghostscript 		*/
+/* and tell ghostscript to run the requested file	*/
+/*------------------------------------------------------*/
+
+#ifndef HAVE_CAIRO
+void write_scale_position_and_run_gs(float norm, float xpos, float ypos,
+      const char *bgfile)
+{
+   send_to_gs("/GSobj save def\n");
+   send_to_gs("/setpagedevice {pop} def\n");
+   send_to_gs("gsave\n");
+   sprintf(_STR, "%3.2f %3.2f translate\n", xpos, ypos);
+   send_to_gs(_STR);
+   sprintf(_STR, "%3.2f %3.2f scale\n", norm, norm);
+   send_to_gs(_STR);
+   sprintf(_STR, "(%s) run\n", bgfile);
+   send_to_gs(_STR);
+   send_to_gs("GSobj restore\n");
+   send_to_gs("grestore\n");
+
+   Wprintf("Rendering background image.");
+   XDefineCursor(dpy, areawin->window, WAITFOR);
+}
+#endif /* !HAVE_CAIRO */
 
 /*------------------------------------------------------*/
 /* Call ghostscript to render the background into the	*/
@@ -634,7 +639,9 @@ int renderbackground()
    float psnorm, psxpos, psypos, defscale;
    float devres = 0.96; /* = 72.0 / 75.0, ps_units/in : screen_dpi */
 
+#ifndef HAVE_CAIRO
    if (gsproc < 0) return -1;
+#endif /* !HAVE_CAIRO */
 
    defscale = (xobjs.pagelist[areawin->page]->coordstyle == CM) ?
 	CMSCALE : INCHSCALE;
@@ -642,8 +649,10 @@ int renderbackground()
    psnorm = areawin->vscale * (1.0 / defscale) * devres;
 
    psxpos = (float)(-areawin->pcorner.x) * areawin->vscale * devres;
-   psypos = (float)(-areawin->pcorner.y) * areawin->vscale * devres
-		+ ((float)areawin->height / 12.0);
+   psypos = (float)(-areawin->pcorner.y) * areawin->vscale * devres;
+#ifndef HAVE_CAIRO
+   psypos += areawin->height / 12.;
+#endif /* !HAVE_CAIRO */
 
    /* Conditions for re-rendering:  Must have a background specified */
    /* and must be on the page, not a library or other object.	     */
@@ -669,30 +678,19 @@ int renderbackground()
 
    areawin->lastbackground = NULL;
 
-   /* write scale and position to ghostscript 		*/
-   /* and tell ghostscript to run the requested file	*/
-
-   send_to_gs("/GSobj save def\n");
-   send_to_gs("/setpagedevice {pop} def\n");
-   send_to_gs("gsave\n");
-   sprintf(_STR, "%3.2f %3.2f translate\n", psxpos, psypos);
-   send_to_gs(_STR);
-   sprintf(_STR, "%3.2f %3.2f scale\n", psnorm, psnorm);
-   send_to_gs(_STR);
-   sprintf(_STR, "(%s) run\n", bgfile);
-   send_to_gs(_STR);
-   send_to_gs("GSobj restore\n");
-   send_to_gs("grestore\n");
-
 #ifdef GS_DEBUG
    fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
 #endif
    Wprintf("Rendering background image.");
-   XDefineCursor(dpy, areawin->window, WAITFOR);
+
+   /* write scale and position to ghostscript 		*/
+   /* and tell ghostscript to run the requested file	*/
+
+   write_scale_position_and_run_gs(psnorm, psxpos, psypos, bgfile);
 
    /* The page will be refreshed when we receive confirmation	*/
    /* from ghostscript that the buffer has been rendered.	*/
-
+   
    return 0;
 }
 
@@ -710,11 +708,11 @@ int copybackground()
    if (is_page(topobject) == -1)
       return -1;
 
-#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
-   XCopyArea(dpy, bbuf, dbuf, areawin->gc, 0, 0,
-             areawin->width, areawin->height, 0, 0);
+#ifdef HAVE_CAIRO
+   cairo_set_source_surface(areawin->cr, bbuf, 0., 0.);
+   cairo_paint(areawin->cr);
 #else
-   XCopyArea(dpy, bbuf, areawin->window, areawin->gc, 0, 0,
+   XCopyArea(dpy, bbuf, dbuf, areawin->gc, 0, 0,
              areawin->width, areawin->height, 0, 0);
 #endif
 
@@ -727,6 +725,7 @@ int copybackground()
 
 int exit_gs()
 {
+#ifndef HAVE_CAIRO
 #ifdef _MSC_VER
    if (gsproc == INVALID_HANDLE_VALUE) return -1;	/* gs not running */
 #else
@@ -752,8 +751,9 @@ int exit_gs()
 #else
    gsproc = -1;
 #endif
-   gs_state = GS_INIT;
+#endif /* !HAVE_CAIRO */
 
+   gs_state = GS_INIT;
    return 0;
 }
 
@@ -763,19 +763,19 @@ int exit_gs()
 
 int reset_gs()
 {
-#ifdef _MSC_VER
+#ifndef HAVE_CAIRO
+ #ifdef _MSC_VER
    if (gsproc == INVALID_HANDLE_VALUE) return -1;
-#else
+ #else
    if (gsproc < 0) return -1;
-#endif
+ #endif
 
    exit_gs();
    ghostinit_local();
    start_gs();
 
+#endif /* !HAVE_CAIRO */
    return 0;
 }
 
-#endif /* !OPENGL */
-
 /*----------------------------------------------------------------------*/
