@@ -1035,7 +1035,7 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
 			tmpext.ascent : (tmpext.ascent + tmpext.base) >> 1)
 			: tmpext.base);
-	   
+
 	   /* Label selection:  For each character in the label string, */
 	   /* do an insideness test with the select box.		*/
 
@@ -1069,6 +1069,19 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 	      }
 	   }
 	   if (state == 1) areawin->textpos = j;   /* selection goes to end of string */
+
+	   /* If a label happens to be empty (can happen in the case of	*/
+	   /* a label with parameters that are all empty strings), then	*/
+	   /* check if the bounding box surrounds the label position.	*/
+
+	   else if (tmpext.width == 0) {
+	      isect = test_insideness(0, 0, newboxpts);
+	      if (isect) {
+		 selected = True;
+		 areawin->textend = 1;
+	      }
+	   }
+	  
 	   } break;
 	   
 	case(PATH):
