@@ -106,7 +106,6 @@ Cursor	appcursors[NUM_CURSORS];
 ApplicationData appdata;
 XCWindowData *areawin;
 Globaldata xobjs;
-int *appcolors;
 int number_colors;
 colorindex *colorlist;
 short menusize;
@@ -129,7 +128,7 @@ extern Tcl_Interp *xcinterp;
 extern XtAppContext app;
 #endif
 
-#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
+#if !defined(HAVE_CAIRO)
 extern Pixmap dbuf;
 #endif
 
@@ -169,15 +168,12 @@ void addtocolorlist(xcWidget button, int cvalue)
 
 int addnewcolorentry(int ccolor)
 {
-  xcWidget newbutton; /* colormenu, (jdk) */
-#ifndef XC_WIN32
-  /* Arg wargs[2]; (jdk) */
-#endif
-  int i; /* , n = 0; (jdk) */
+  xcWidget newbutton = (xcWidget)NULL;
+  int i;
 
    /* check to see if entry is already in the color list */
 
-   for (i = 0; i < number_colors; i++)
+   for (i = NUMBER_OF_COLORS; i < number_colors; i++)
       if (colorlist[i].color.pixel == ccolor) break;
 
    /* make new entry in the menu */
@@ -687,33 +683,22 @@ int findnearcolor(XColor *cvexact)
 
 #endif
 
-/*------------------------------------------------------------------------------*/
-/* Layout colors which are defined from the Xdefaults (BBOXCOLOR, schematic	*/
-/* label colors) may be duplicated in the colormap.  This routine takes a color	*/
-/* index of a color and returns the index of any color in the layout colormap	*/
-/* which matches it within the tolerance used by "rgb_alloccolor".  This will	*/
-/* be used to remap the Xdefault-supplied colors to the layout color table, so 	*/
-/* we don't get duplicate entries.						*/
-/*------------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Return the pixel value of a database color in the colorlist vector	*/
+/* Since this is generally called by defined name, e.g.,		*/
+/* xc_getlayoutcolor(RATSNESTCOLOR), it does not check for invalid	*/
+/* values of cidx.							*/
+/*----------------------------------------------------------------------*/
 
 int xc_getlayoutcolor(int cidx)
 {
-   XColor loccolor;
-   int locidx; /* i, (jdk) */
-
-   loccolor.pixel = cidx;
-   loccolor.flags = DoRed | DoGreen | DoBlue;
-
-   XQueryColors(dpy, cmap, &loccolor, 1);
-
-   locidx = rgb_alloccolor(loccolor.red, loccolor.green, loccolor.blue);
-   return locidx;
+   return colorlist[cidx].color.pixel;
 }
 
-/*-------------------------------------------------------------------------*/
-/* Obtain red, green and blue values from a color index                    */
-/* the colour values range from 0 to 65535                                 */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Obtain red, green and blue values from a color index                 */
+/* the colour values range from 0 to 65535                              */
+/*----------------------------------------------------------------------*/
 
 void xc_get_color_rgb(unsigned long cidx, unsigned short *red, 
       unsigned short *green, unsigned short *blue)
@@ -818,7 +803,7 @@ int rgb_querycolor(int red, int green, int blue, int *pixval)
 {
    int i;
 
-   for (i = 0; i < number_colors; i++) {
+   for (i = NUMBER_OF_COLORS; i < number_colors; i++) {
       if (abs(colorlist[i].color.red - red) < 512 &&
 	     abs(colorlist[i].color.green - green) < 512 &&
 	     abs(colorlist[i].color.blue - blue) < 512) {
@@ -839,6 +824,10 @@ int rgb_querycolor(int red, int green, int blue, int *pixval)
 
 int rgb_alloccolor(int red, int green, int blue)
 {
+   ERROR FIXME!
+   // XCircuit version 3.9:  This no longer works.  Need to
+   // allocate the color and return an index into colorlist[].
+
    return RGB(red >> 8, green >> 8, blue >> 8);
 }
 
@@ -863,8 +852,11 @@ int rgb_alloccolor(int red, int green, int blue)
          pixval = findnearcolor(&newcolor);
       else
 	 pixval = newcolor.pixel;
+
+      // Add this to the colormap
+      tableidx = addnewcolorentry(pixval);
    }
-   return pixval;
+   return tableidx;
 }
 
 #endif
@@ -900,8 +892,8 @@ void makecursors()
    Tk_Uid fg_uid, bg_uid;
 #endif
 
-   bgcolor.pixel = BACKGROUND;
-   fgcolor.pixel = FOREGROUND;
+   bgcolor.pixel = colorlist[BACKGROUND].color.pixel;
+   fgcolor.pixel = colorlist[FOREGROUND].color.pixel;
    XQueryColors(dpy, cmap, &fgcolor, 1);
    XQueryColors(dpy, cmap, &bgcolor, 1);
 
@@ -1092,17 +1084,22 @@ XCWindowData *create_new_window()
    newwindow->attachto = -1;
    newwindow->color = DEFAULTCOLOR;
    newwindow->gccolor = 0;
-   newwindow->gctype = 0;
    newwindow->time_id = 0;
-#ifndef HAVE_CAIRO
+   newwindow->redraw_needed = True;
+   newwindow->redraw_ongoing = False;
+#ifdef HAVE_CAIRO
+   newwindow->fixed_pixmap = NULL;
+#else /* HAVE_CAIRO */
    newwindow->clipmask = (Pixmap)NULL;
    newwindow->pbuf = (Pixmap)NULL;
    newwindow->cmgc = (GC)NULL;
    newwindow->clipped = 0;
+   newwindow->fixed_pixmap = (Pixmap) NULL;
 #endif /* !HAVE_CAIRO */
    newwindow->vscale = 1;
    newwindow->pcorner.x = newwindow->pcorner.y = 0;
    newwindow->topinstance = (objinstptr)NULL;
+   newwindow->panx = newwindow->pany = 0;
 
    /* Prepend to linked window list in global data (xobjs) */
    newwindow->next = xobjs.windowlist;
@@ -1471,21 +1468,20 @@ void post_initialize()
    /* set in initmem() should correctly position the empty page in the	 */
    /* middle of the viewing window.					 */
 
-#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
+#if !defined(HAVE_CAIRO)
    if (dbuf == (Pixmap)NULL)
       dbuf = XCreatePixmap(dpy, areawin->window, areawin->width,
 		areawin->height, DefaultDepthOfScreen(xcScreen(areawin->area)));
 #endif
 
-#ifdef TCL_WRAPPER
-
    /* Set up fundamentally necessary colors black and white */
 
    addnewcolorentry(xc_alloccolor("Black"));
    addnewcolorentry(xc_alloccolor("White"));
 
-   /* Set up new Tcl type "handle" for element handles */
+#ifdef TCL_WRAPPER
 
+   /* Set up new Tcl type "handle" for element handles */
    Tcl_RegisterObjType(&tclHandleType);
 
 #endif
