@@ -28,18 +28,21 @@ typedef unsigned long long	u_long;
 #endif
 
 /*----------------------------------------------------------------------*/
-/* Graphics functions defined for X11 (i.e., not OpenGL)		*/
+/* portable definition to prevent unused variables warning		*/
 /*----------------------------------------------------------------------*/
 
-#ifndef OPENGL
+#define UNUSED(x) (void) x
+
+/*----------------------------------------------------------------------*/
+/* Graphics functions defined for X11					*/
+/*----------------------------------------------------------------------*/
 
 #ifdef HAVE_CAIRO
 
 #define SetForeground(display, gc, fg) xc_cairo_set_color(fg)
-#define SetFunction(display, gc, function)
 typedef cairo_surface_t xcImage;
 
-#else /* HAVE_CAIRO */
+#else /* !HAVE_CAIRO */
 
 typedef XImage xcImage;
 #define DrawLine	XDrawLine
@@ -47,9 +50,8 @@ typedef XImage xcImage;
 #define DrawPoint	XDrawPoint
 #define FillPolygon	XFillPolygon
 
-#define SetForeground	XSetForeground
-#define SetBackground	XSetBackground
-#define SetFunction	XSetFunction
+#define SetForeground(dpy, gc, fg)  XSetForeground(dpy, gc, colorlist[fg].color.pixel)
+#define SetBackground(dpy, gc, bg)  XSetBackground(dpy, gc, colorlist[bg].color.pixel)
 #define SetThinLineAttributes	XSetLineAttributes
 #define SetLineAttributes(a, b, c, d, e, f) \
 	 XSetLineAttributes(a, b, ((c) >= 1.55 ? (int)(c + 0.45) : 0), d, e, f)
@@ -57,17 +59,10 @@ typedef XImage xcImage;
 #define SetFillStyle	XSetFillStyle
 #define SetStipple(a,b,c) XSetStipple(a,b,STIPPLE[c])
 
-#endif /* HAVE_CAIRO */
+#endif /* !HAVE_CAIRO */
 
 #define flusharea()	
 
-#else
-
-#define SetThinLineAttributes	SetLineAttributes
-#define flusharea	glFlush
-
-#endif /* !OPENGL */
-
 /*----------------------------------------------------------------------*/
 /* Redefinition of fprintf() allows redirection of output to a console	*/
 /* in the Tcl interpreter-based version.				*/
@@ -84,7 +79,7 @@ typedef XImage xcImage;
 #ifdef TCL_WRAPPER
   #define malloc Tcl_Alloc
   /* (see definition of my_calloc in the asg subdirectory) */
-  // #define calloc(a,b) Tcl_Alloc(a * b)
+  /* #define calloc(a,b) Tcl_Alloc(a * b) */
   #define free(a) Tcl_Free((char *)(a))
   #define realloc(a,b) Tcl_Realloc((char *)(a), b)
   #undef strdup
@@ -339,14 +334,13 @@ typedef struct {
 /* Event mode definitions (state of drawing area)			*/
 /*----------------------------------------------------------------------*/
 
-enum editmode {
+typedef enum editmode {
   NORMAL_MODE = 0,	/* On the drawing page, none of the situations below */
   UNDO_MODE,		/* In the process of an undo/redo operation */
   MOVE_MODE,		/* In the process of moving elements */
   COPY_MODE,		/* In the process of copying elements */
   PAN_MODE,		/* In the process of panning to follow the cursor */
   SELAREA_MODE,		/* Area selection box */
-  PENDING_MODE,		/* Temporary mode to select without drawing selection */
   RESCALE_MODE,		/* Interactive element rescaling box */
   CATALOG_MODE,		/* On a library page, library directory, or page directory */
   CATTEXT_MODE,		/* Editing an existing object name in the library */
@@ -365,7 +359,7 @@ enum editmode {
   EINST_MODE,		/* Editing an instance (from the level above) */
   ASSOC_MODE,		/* Choosing an associated schematic or symbol */
   CATMOVE_MODE		/* Moving objects in or between libraries */
-};
+} event_mode_t;
 
 /*----------------------------------------------------------------------*/
 /* File loading modes							*/
@@ -801,7 +795,7 @@ typedef struct {
    float	  tscale;	/* target scale (0 = uninitialized) */
    Pixmap	  clipmask;	/* clipmask for non-manhattan rotations */
    Boolean	  valid;	/* does target need to be regenerated? */
-#endif /* HAVE_CAIRO */
+#endif /* !HAVE_CAIRO */
 } graphic, *graphicptr;
 
 /*----------------------------------------------------------------------*/
@@ -1223,31 +1217,12 @@ typedef struct {
 #define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
 	areawin->gc, FOREGROUND); else SetForeground(dpy, areawin->gc, a)
 
-#define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
-	areawin->gctype = z
-
 #define XcTopSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
 #define XcSetForeground(z) SetForeground(dpy, areawin->gc, z); \
 	areawin->gccolor = z
 
-#ifdef HAVE_CAIRO
-
-   #define XSetXORFg(a,b) XTopSetForeground(a)
-   #define XcSetXORFg(y,z) XcTopSetForeground(y)
-
-#else /* HAVE_CAIRO */
-
-   #define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-        areawin->gc, FOREGROUND ^ b); else SetForeground(dpy, areawin->gc,\
-        a ^ b)
-
-   #define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
-	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
-
-#endif /* HAVE_CAIRO */
-
 /*----------------------------------------------------------------------*/
 /* Structure for maintaining list of colors				*/
 /*----------------------------------------------------------------------*/
@@ -1280,6 +1255,10 @@ typedef struct {
 #ifdef HAVE_CAIRO
    const char **utf8encoding;
    cairo_font_face_t *font_face;
+   unsigned long glyph_index[256];
+   double glyph_top[256];
+   double glyph_bottom[256];
+   double glyph_advance[256];
 #endif /* HAVE_CAIRO */
 } fontinfo;
 
@@ -1418,6 +1397,7 @@ typedef struct _windowdata {
    /* widgets and X11 parameters */
    xcWidget	area;
    xcWidget	scrollbarh, scrollbarv;
+   int		panx, pany;
    Window	window;
    GC 		gc;
 #ifndef HAVE_CAIRO
@@ -1426,13 +1406,18 @@ typedef struct _windowdata {
    char		clipped;
    GC		cmgc;
 #endif /* !HAVE_CAIRO */
-   int		gccolor, gctype;
+   int		gccolor;
    xcIntervalId time_id;
    Boolean	redraw_needed;
    Boolean	redraw_ongoing;
 #ifdef HAVE_CAIRO
    cairo_surface_t *surface;
    cairo_t	*cr;		/* cairo_drawing context */
+   cairo_pattern_t *fixed_pixmap; /* pixmap holding the background data of   */
+			        /* all fixed element. ie. not including the  */
+			        /* element currently being edited.	     */
+#else /* HAVE_CAIRO */
+   Pixmap       fixed_pixmap;
 #endif /* HAVE_CAIRO */
    /* global page parameters */
    short	width, height;
@@ -1483,7 +1468,7 @@ typedef struct _windowdata {
    Matrixptr	MatStack;
    pushlistptr	stack;
    pushlistptr	hierstack;
-   short	event_mode;
+   event_mode_t	event_mode;
    char		*lastbackground;
    Cursor	*defaultcursor;
 } XCWindowData;
@@ -1565,6 +1550,48 @@ typedef struct {
    XCWindowData *windowlist;	/* linked list of known windows */
 } Globaldata;
 
+
+/*-------------------------------------------------------------------------*/
+/* Track state of the ghostscript renderer 				   */
+/*-------------------------------------------------------------------------*/
+
+typedef enum {
+   GS_INIT,	/* Initial state; gs is idle. */
+   GS_PENDING,	/* Drawing in progress; gs is busy. */
+   GS_READY	/* Drawing done; gs is waiting for "next". */
+} gs_state_t;
+
+
+/*-------------------------------------------------------------------------*/
+/* For the eventmodes (ARC_MODE, EARC_MODE, SPLINE_MODE, etc) the drawing  */
+/* and/or redrawing of the elements is centralized in functions		   */
+/* corresponding to the eventmode name (arc_mode_draw, spline_mode_draw,   */
+/* etc.). Depending on the state these functions clear, draw or whatever   */
+/* is needed. A description of the options follows below		   */
+/* fixed_pixmap refers to the background pixmap without the element(s)     */
+/* currently being edited.						   */
+/*-------------------------------------------------------------------------*/
+
+typedef enum {
+   xcDRAW_INIT,		/* Initalize fixed_pixmap and draw the element(s)  */
+			/* element(s) currently being edited, including	   */
+			/* edit lines.					   */
+
+   xcDRAW_EDIT,		/* Draw the currently edited element(s), including */
+			/* edit lines.					   */
+   
+   xcDRAW_FINAL,	/* Draw the finalized element(s), without edit	   */
+			/* lines to the fixed_pixmap			   */
+
+   xcDRAW_EMPTY,	/* Refresh the screen by drawing the fixed_pixmap  */
+			/* This essentially clear the edited elements      */
+
+   xcREDRAW_FORCED	/* Redraws everything, including edit lines. Only  */
+	   		/* used in drawarea. This one should be called     */
+			/* when something has changed (eg. zoom).	   */
+			/* It will redraw the fixed_pixmap		   */
+} xcDrawType;
+
 /*----------------------------------------------------------------------*/
 /* structures previously defined in menudefs.h				*/
 /*----------------------------------------------------------------------*/
@@ -1602,3 +1629,4 @@ typedef struct _toolbarstruct {
 /* Menus and Toolbars are taken care of entirely by scripts in the Tcl/	*/
 /* Tk version of xcircuit.						*/
 /*----------------------------------------------------------------------*/
+
