@@ -46,7 +46,6 @@ extern xcWidget	 top;
 extern Display   *dpy;
 extern Globaldata xobjs;
 extern XCWindowData *areawin;
-extern int	  *appcolors;
 extern int	  number_colors;
 extern colorindex *colorlist;
 extern ApplicationData appdata;
@@ -66,11 +65,11 @@ extern short	 popups;	     /* total number of popup windows */
 #include "menudep.h"
 
 /*----------------------------------------------------------------------*/
-/* Local Variable definitions						*/
+/* External variable definitions					*/
 /*----------------------------------------------------------------------*/
 
-u_short *fontnumbers;
-u_char nfontnumbers;
+extern u_short *fontnumbers;
+extern u_char nfontnumbers;
 
 /*----------------------------------------------*/
 /* Set Poly and Arc line styles and fill styles */
@@ -414,7 +413,7 @@ buttonsave *getgeneric(xcWidget button, void (*getfunction)(), void *dataptr)
    if (button != NULL) {
       XtSetArg(wargs[0], XtNforeground, &saveptr->foreground);
       XtGetValues(button, wargs, 1);
-      XtSetArg(wargs[0], XtNforeground, OFFBUTTONCOLOR);
+      XtSetArg(wargs[0], XtNforeground, colorlist[OFFBUTTONCOLOR].color.pixel);
       XtSetValues(button, wargs, 1);
       XtRemoveAllCallbacks(button, XtNselect);
    }
@@ -1116,8 +1115,8 @@ void highlightexcl(xcWidget widget, int func, int value)
 
    /* remove highlight from all widgets in the toolbar */
 
-   XtSetArg(args[0], XtNbackground, BACKGROUND);
-   XtSetArg(args[1], XtNborderColor, SNAPCOLOR);
+   XtSetArg(args[0], XtNbackground, colorlist[BACKGROUND].color.pixel);
+   XtSetArg(args[1], XtNborderColor, colorlist[SNAPCOLOR].color.pixel);
    for (titem = ToolBar; titem < ToolBar + toolbuttons; titem++) {
       sibling = XtNameToWidget(parent, titem->name);
       if (sibling == widget)
@@ -1155,8 +1154,8 @@ void highlightexcl(xcWidget widget, int func, int value)
    /* Now highlight the currently pushed widget */
 
    if (self != (xcWidget)NULL) {
-      XtSetArg(args[0], XtNbackground, RATSNESTCOLOR);
-      XtSetArg(args[1], XtNborderColor, RATSNESTCOLOR);
+      XtSetArg(args[0], XtNbackground, colorlist[RATSNESTCOLOR].color.pixel);
+      XtSetArg(args[1], XtNborderColor, colorlist[RATSNESTCOLOR].color.pixel);
       XtSetValues(self, args, 2);
    }
 }
@@ -1405,18 +1404,13 @@ void setcolormark(int colorval)
 
    if (colorval == DEFAULTCOLOR)
       w = ColorInheritColorButton;
-   else {
-      for (i = 0; i < number_colors; i++)
-         if (colorlist[i].color.pixel == colorval) {
-            w = colorlist[i].cbutton;
-	    break;
-         }
-   }
+   else
+      w = colorlist[colorval].cbutton;
 
    /* Remove mark from all menu items */
 
    XtSetArg(args[0], XtNsetMark, False);
-   for (i = 0; i < number_colors; i++)
+   for (i = NUMBER_OF_COLORS; i < number_colors; i++)
       XtSetValues(colorlist[i].cbutton, args, 1);
    XtSetValues(ColorInheritColorButton, args, 1);
 
@@ -1539,7 +1533,7 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
       XtSetArg(wargs[0], XtNrectColor, &cval);
       XtGetValues(w, wargs, 1);
 
-      for (cindex = 0; cindex < number_colors; cindex++)
+      for (cindex = NUMBER_OF_COLORS; cindex < number_colors; cindex++)
          if (colorlist[cindex].color.pixel == cval)
 	    break;
       if (cindex >= number_colors) {
@@ -1565,7 +1559,7 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
 	    redrawtext(curlabel);
       }
       else {
-	 // sprintf(_STR2, "%d", cindex);
+	 sprintf(_STR2, "%d", cindex);
 	 labeltext(FONT_COLOR, (char *)&cindex);
       }
    }
@@ -1575,19 +1569,17 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
 	   + areawin->selects; scolor++) {
 	 ecolor = &(SELTOCOLOR(scolor));
 
-	 *ecolor = cval;
+	 *ecolor = cindex;
 	 selected = True;
       }
    }
 
-   setcolormark(cval);
+   setcolormark(cindex);
    if (!selected) {
       if (eventmode != TEXT_MODE && eventmode != ETEXT_MODE)
-         areawin->color = cval;      
+         areawin->color = cindex;      
       overdrawpixmap(w);
    }
-
-   unselect_all();
 }
 
 /*----------------------------------------------------------------*/
@@ -1618,10 +1610,10 @@ void setnewcolor(xcWidget w, caddr_t nullptr)
       }
       ccolor = rgb_alloccolor(red, green, blue);
    }
-   else
+   else {
       ccolor = xc_alloccolor(_STR2);
-
-   addnewcolorentry(ccolor);
+      addnewcolorentry(ccolor);
+   }
 }
 
 /*----------------------------------------------------------------*/
